@@ -76,7 +76,7 @@ class PathTestCase(unittest.TestCase):
         path = Path('//*')
         self.assertEqual('<Path "descendant-or-self::node()/child::*">',
                          repr(path))
-        self.assertEqual('<elem/>', path.select(xml).render())
+        self.assertEqual('<root><elem/></root>', path.select(xml).render())
 
     def test_1step_attribute(self):
         path = Path('@foo')
@@ -328,6 +328,11 @@ class PathTestCase(unittest.TestCase):
         path = Path('*[contains(name(), "oo")]')
         self.assertEqual('<foo>bar</foo>', path.select(xml).render())
 
+    def test_predicate_matches_function(self):
+        xml = XML('<root><foo>bar</foo><bar>foo</bar></root>')
+        path = Path('*[matches(name(), "foo|bar")]')
+        self.assertEqual('<foo>bar</foo><bar>foo</bar>', path.select(xml).render())
+
     def test_predicate_false_function(self):
         xml = XML('<root><foo>bar</foo></root>')
         path = Path('*[false()]')
